@@ -7,21 +7,54 @@ use File::Glob qw( bsd_glob );
 use ExtUtils::MakeMaker ();
 use IPC::Cmd ();
 use lib 'inc';
+use File::Spec;
+use My::BuildConfig;
 use My::ShareConfig;
+use Capture::Tiny qw( capture_merged );
+
+{
+  my $dh;
+  opendir $dh, 'inc';
+  my @files = map { File::Spec->catfile('inc', $_) } grep /^bad-.*\.pl$/, readdir $dh;
+  close $dh;
+
+  foreach my $badcheck (@files)
+  {
+    my($out, $ret) = capture_merged {
+      system $^X, $badcheck;
+      $?;
+    };
+    if($ret)
+    {
+      if($out ne '')
+      {
+        print $out;
+        exit;
+      }
+      else
+      {
+        print "bad check $badcheck failed\n";
+        exit;
+      }
+    }
+  }
+}
 
 sub myWriteMakefile
 {
   my %args = @_;
+  my $build_config = My::BuildConfig->new;
   my $share_config = My::ShareConfig->new;
   my %diag;
   my %alien;
 
   ExtUtils::MakeMaker->VERSION('7.12');
+  $build_config->set(version => [ $args{VERSION} =~ /^([0-9]+)\.([0-9]{2})/ ]);
 
   if(eval { require Alien::FFI; Alien::FFI->VERSION('0.20'); 1 })
   {
     print "using already installed Alien::FFI (version @{[ Alien::FFI->VERSION ]})\n";
-    $share_config->set(alien => { class => 'Alien::FFI', mode => 'already-installed' });
+    $build_config->set(alien => { class => 'Alien::FFI', mode => 'already-installed' });
     require Alien::Base::Wrapper;
     Alien::Base::Wrapper->import( 'Alien::FFI', 'Alien::psapi', '!export' );
     %alien = Alien::Base::Wrapper->mm_args;
@@ -30,18 +63,21 @@ sub myWriteMakefile
   {
     require Alien::FFI::pkgconfig    if $^O ne 'MSWin32';
     require Alien::FFI::PkgConfigPP  if $^O eq 'MSWin32';
-    if($^O eq 'MSWin32' && Alien::FFI::PkgConfigPP->exists)
+
+    my $alien_install_type_unset = !defined $ENV{ALIEN_INSTALL_TYPE};
+
+    if($alien_install_type_unset && $^O eq 'MSWin32' && Alien::FFI::PkgConfigPP->exists)
     {
       print "using system libffia via PkgConfigPP\n";
-      $share_config->set(alien => { class => 'Alien::FFI::PkgConfigPP', mode => 'system' });
+      $build_config->set(alien => { class => 'Alien::FFI::PkgConfigPP', mode => 'system' });
       require Alien::Base::Wrapper;
       Alien::Base::Wrapper->import( 'Alien::FFI::PkgConfigPP', 'Alien::psapi', '!export' );
       %alien = Alien::Base::Wrapper->mm_args;
     }
-    elsif($^O ne 'MSWin32' && Alien::FFI::pkgconfig->exists)
+    elsif($alien_install_type_unset && $^O ne 'MSWin32' && Alien::FFI::pkgconfig->exists)
     {
       print "using system libffi via @{[ Alien::FFI::pkgconfig->pkg_config_exe ]}\n";
-      $share_config->set(alien => { class => 'Alien::FFI::pkgconfig', mode => 'system' });
+      $build_config->set(alien => { class => 'Alien::FFI::pkgconfig', mode => 'system' });
       require Alien::Base::Wrapper;
       Alien::Base::Wrapper->import( 'Alien::FFI::pkgconfig', 'Alien::psapi', '!export' );
       %alien = Alien::Base::Wrapper->mm_args;
@@ -49,7 +85,7 @@ sub myWriteMakefile
     else
     {
       print "requiring Alien::FFI in fallback mode.\n";
-    $share_config->set(alien => { class => 'Alien::FFI', mode => 'fallback' });
+    $build_config->set(alien => { class => 'Alien::FFI', mode => 'fallback' });
       %alien = (
         CC => '$(FULLPERL) -Iinc -MAlien::Base::Wrapper=Alien::FFI,Alien::psapi -e cc --',
         LD => '$(FULLPERL) -Iinc -MAlien::Base::Wrapper=Alien::FFI,Alien::psapi -e ld --',
@@ -71,14 +107,14 @@ sub myWriteMakefile
     print "DEBUG_FAKE32:\n";
     print "  + making Math::Int64 a prereq\n";
     print "  + Using Math::Int64's C API to manipulate 64 bit values\n";
-    $share_config->set(config_debug_fake32 => 1);
+    $build_config->set(config_debug_fake32 => 1);
     $diag{config}->{config_debug_fake32} = 1;
   }
   if($ENV{FFI_PLATYPUS_NO_ALLOCA})
   {
     print "NO_ALLOCA:\n";
     print "  + alloca() will not be used, even if your platform supports it.\n";
-    $share_config->set(config_no_alloca => 1);
+    $build_config->set(config_no_alloca => 1);
     $diag{config}->{config_no_alloca} = 1;
   }
 
@@ -108,7 +144,7 @@ sub myWriteMakefile
   # uniq'ify it
   @dlext = do { my %seen; grep { !$seen{$_}++ } @dlext };
 
-  $share_config->set(diag => \%diag);
+  $build_config->set(diag => \%diag);
   $share_config->set(config_dlext => \@dlext);
 
   ExtUtils::MakeMaker::WriteMakefile(%args);
@@ -197,8 +233,8 @@ sub postamble {
 
   foreach my $key (qw( cc inc ccflags cccdlflags optimize ld ldflags lddlflags ))
   {
-    $postamble .= 
-      sprintf "\t$noecho\$(FULLPERL) inc/mm-config-set.pl eumm.%-20s \$(%s)\n", $key, uc $key;
+    $postamble .=
+      sprintf "\t$noecho\$(FULLPERL) inc/mm-config-set.pl %-20s \$(%s)\n", $key, uc $key;
   }
 
   $postamble .=
