@@ -335,6 +335,14 @@ typedef struct _ffi_pl_heap {
   heap = n;                                 \
 }
 
+#define ffi_pl_heap_add_ptr(ptr) {          \
+  ffi_pl_heap *n;                           \
+  Newx(n, 1, ffi_pl_heap);                  \
+  n->_this = ptr;                           \
+  n->_next = (void*) heap;                  \
+  heap = n;                                 \
+}
+
 #define ffi_pl_heap_free() {                \
   while(heap != NULL)                       \
   {                                         \
@@ -345,6 +353,9 @@ typedef struct _ffi_pl_heap {
   }                                         \
 }
 
+#define ffi_pl_croak                        \
+  ffi_pl_heap_free();                       \
+  croak
 
 #if defined(_MSC_VER)
 #define Newx_or_alloca(ptr, count, type) ptr = _alloca(sizeof(type)*count)
@@ -357,6 +368,18 @@ typedef struct _ffi_pl_heap {
 ffi_type *ffi_pl_type_to_libffi_type(ffi_pl_type *type);
 ffi_pl_type *ffi_pl_type_new(size_t size);
 
+#if SIZEOF_VOIDP == 4
+uint64_t cast0(void);
+#else
+void *cast0(void);
+#endif
+
+#if SIZEOF_VOIDP == 4
+uint64_t cast1(uint64_t value);
+#else
+void *cast1(void *value);
+#endif
+
 #ifdef __cplusplus
 }
 #endif
